use std::error::Error;
use std::path::Path;

use clap::Parser;
use image::{GenericImageView, Rgba, RgbaImage};
use walkdir::WalkDir;

/// This program recursively finds all PNG files in a given directory and
/// replaces each with a new image that:
/// - has the same name and dimensions as the original,
/// - is filled with a solid color equal to the average RGB of the original image’s non-transparent pixels,
/// - preserves the transparency of the original.
/// 
/// You can also provide an optional multiplier for the RGB values as three numbers.
#[derive(Parser, Debug)]
#[command(author, version, about)]
struct Args {
    /// Path of the directory to process
    directory: String,

    /// Optional multiplier for RGB values (provide three numbers, e.g., --multiplier 1.0 0.5 1.0)
    #[arg(short, long, value_parser = clap::value_parser!(f32), num_args = 3)]
    multiplier: Option<Vec<f32>>,
}

fn main() -> Result<(), Box<dyn Error>> {
    let args = Args::parse();
    // If a multiplier is provided, convert the Vec into a tuple (r, g, b)
    let multiplier = args.multiplier.map(|v| (v[0], v[1], v[2]));

    // Walk through the directory recursively.
    for entry in WalkDir::new(&args.directory).into_iter().filter_map(Result::ok) {
        if entry.file_type().is_file() {
            let path = entry.path();
            // Check if the file extension is png (case-insensitive)
            if path.extension().map_or(false, |ext| ext.eq_ignore_ascii_case("png")) {
                println!("Processing: {:?}", path);
                process_image(path, multiplier)?;
            }
        }
    }
    Ok(())
}

/// Processes a single image:
/// - Computes the average RGB of non-transparent pixels.
/// - Optionally applies an RGB multiplier.
/// - Creates a new image that uses the computed color in place of all non-transparent pixels,
///   while preserving each pixel’s original alpha.
fn process_image(path: &Path, multiplier: Option<(f32, f32, f32)>) -> Result<(), Box<dyn Error>> {
    // Open the image file
    let img = image::open(path)?;
    // Convert to an RGBA8 image for easy per-pixel manipulation.
    let img_rgba = img.to_rgba8();
    let (width, height) = img_rgba.dimensions();

    // Calculate the average color of non-transparent pixels.
    let (mut sum_r, mut sum_g, mut sum_b, mut count) = (0u64, 0u64, 0u64, 0u64);
    for pixel in img_rgba.pixels() {
        let [r, g, b, a] = pixel.0;
        if a != 0 {
            sum_r += r as u64;
            sum_g += g as u64;
            sum_b += b as u64;
            count += 1;
        }
    }
    let (avg_r, avg_g, avg_b) = if count > 0 {
        (
            (sum_r / count) as u8,
            (sum_g / count) as u8,
            (sum_b / count) as u8,
        )
    } else {
        (0, 0, 0)
    };

    // Apply the multiplier if provided.
    let (final_r, final_g, final_b) = if let Some((mr, mg, mb)) = multiplier {
        (
            ((avg_r as f32 * mr).min(255.0)).round() as u8,
            ((avg_g as f32 * mg).min(255.0)).round() as u8,
            ((avg_b as f32 * mb).min(255.0)).round() as u8,
        )
    } else {
        (avg_r, avg_g, avg_b)
    };

    // Create a new image with the same dimensions.
    let mut new_img = RgbaImage::new(width, height);
    // For every pixel in the original image:
    // - If the pixel is transparent (alpha == 0), leave it transparent.
    // - Otherwise, use the computed solid color while preserving the original alpha.
    for (x, y, pixel) in img_rgba.enumerate_pixels() {
        let a = pixel.0[3];
        if a == 0 {
            new_img.put_pixel(x, y, Rgba([0, 0, 0, 0]));
        } else {
            new_img.put_pixel(x, y, Rgba([final_r, final_g, final_b, a]));
        }
    }

    // Overwrite the original image file with the new image.
    new_img.save(path)?;

    Ok(())
}
